require "net/http"
require 'net/http/post/multipart'
require 'yaml'

module Anamo
  class Api

    def initialize
      @api_base_url = 'https://app.anamo.io/api'
      @config = YAML.load_file "#{Dir.home}/anamo.conf.yml"
      @client_key = File.exists?("#{Dir.home}/anamo.client.key") ? File.open("#{Dir.home}/anamo.client.key", 'rb') { |f| f.read } : nil
    end

    def make_url path
      "#{@api_base_url}/#{path}"
    end

    def make_uri path
      URI.parse make_url path
    end

    def do_post path, &block
      uri = make_uri path
      http = Net::HTTP.new(uri.host, uri.port)
      http.use_ssl = true
      request = Net::HTTP::Post.new(uri.request_uri)
      request['X-UPTB-Client-Key'] = @client_key if @client_key
      instance_exec request, http, &block
      http.request(request)
    end

    def do_multipart_post path, files
      uri = make_uri path
      request = Net::HTTP::Post::Multipart.new uri.path, files
      request['X-UPTB-Client-Key'] = @client_key if @client_key
      Net::HTTP.start(uri.host, uri.port, :use_ssl => true) do |http|
        http.request(request)
      end
    end

    def post_fstree data_files
      response = do_multipart_post 'fstree', data_files
      raise 'Send failure' if !response.is_a?(Net::HTTPOK)
      response
    end

    def post_users data
      response = do_post 'users' do |request|
        request.body = data
        request["Content-Type"] = "text/plain"
      end
      raise 'Send failure' if !response.is_a?(Net::HTTPOK)
      response
    end

    def post_groups data
      response = do_post 'groups' do |request|
        request.body = data
        request["Content-Type"] = "text/plain"
      end
      raise 'Send failure' if !response.is_a?(Net::HTTPOK)
      response
    end

    def post_ports data
      response = do_post 'ports' do |request|
        request.body = data
        request["Content-Type"] = "application/json"
      end
      raise 'Send failure' if !response.is_a?(Net::HTTPOK)
      response
    end

    def post_pkgver data
      response = do_post 'pkgver' do |request|
        request.body = data
        request["Content-Type"] = "application/json"
      end
      raise 'Send failure' if !response.is_a?(Net::HTTPOK)
      response
    end

    def post_node data
      response = do_post 'node' do |request|
        request.body = data
        request["Content-Type"] = "application/json"
      end
      #raise 'Send failure' if !response.is_a?(Net::HTTPOK)
      response
    end

  end
end
